import os

def output(path, iname, counts, qclist):
    imgtotal = sum(qclist)
    total = sum(counts)
    wbc = sum(counts) - counts[8] - counts[7]
    with open(os.path.join(path, iname+'_output.txt'), 'w') as f:
        f.write('Slie quality check\n')
        f.write(f'Total images: {imgtotal}\n')
        f.write(f'Blank: {qclist[0]}\n')
        f.write(f'Cluster: {qclist[1]}\n')
        f.write(f'Standard: {qclist[2]}\n')
        f.write('Cell counts on standard images\n')
        f.write(f'Total: {total} cells\n')
        f.write(f'RBC: {counts[8]} cells\n')
        f.write(f'WBC: {wbc} cells\n')
        f.write(f'Thrombocyte: {counts[7]} cells\n')
        f.write('---WBC count per cell type---\n')
        f.write(f'Neutrophil: {counts[0]} cells\n')
        f.write(f'Eosinphil: {counts[1]} cells\n')
        f.write(f'Blast: {counts[2]} cells\n')
        f.write(f'Erythroid precursor: {counts[3]} cells\n')
        f.write(f'Myeloid precursor: {counts[4]} cells\n')
        f.write(f'Lymphocyte: {counts[5]} cells\n')
        f.write(f'Monocyte+Macrophage: {counts[6]} cells\n')
        f.write(f'Immature monocyte: {counts[9]} cells\n')


import numpy as np
import torch
import torchvision.transforms as transforms

MPATH = '/home/eunhye/fishproj/pipeline_new/models/qcmodel.pkl'

# Predicts image quality class
#    Blank: returnVal == 0
#    Cluster: returnVal == 1
#    Standard: returnVal == 2

def classify(model, inImg, devicenum):
    returnVal = None

    transform = transforms.Compose([
        transforms.ToTensor(),
        transforms.Lambda(lambda x: x[:3, :, :]),
    ])

    inImg = inImg.resize((100, 50))
    inImg = np.array(inImg)
    inImg = transform(inImg)
    inImg = torch.unsqueeze(inImg, 0)
    inImg = inImg.to(devicenum)
    
    with torch.no_grad():
        out = model(inImg)
    _, returnVal = torch.max(out, 1)

    return returnVal


from setuptools import setup, find_packages

requirements = [
    'scikit-build','torch','torchvision','numpy',
    'ultralytics','opencv-python','openslide-python',
    'joblib'
]
    # 'openslide-bin',

setup(
    name='zebrafishblood',
    version='0.3.1',
    description='Zebrafish blood smear cell counter',
    long_description='For python>=3.8',
    author=['Eunhye Yang'],
    author_email='eunhye@connect.hku.hk',
    packages=find_packages(),
    install_requires=requirements
)


# import os
# import sys
# import logging
# import pkg_resources
# from tqdm import tqdm
# import torch
# import joblib
# from ultralytics import YOLO
# from .imgprocessing import *
# from .classify import *
# from .predict import *
# from .output import *

# logging.getLogger('ultralytics').setLevel(logging.WARNING)
# models_dir = pkg_resources.resource_filename(__name__, 'models')
# MPATH = os.path.join(models_dir, 'detection.pt')
# MODEL = YOLO(MPATH)
# QPATH = os.path.join(models_dir, 'qcmodel.pkl')
# home_dir = os.path.expanduser('~')
# ipath = os.path.join(home_dir, 'imagetiles')
# DPATH = ''
# OPATH = os.path.join(home_dir, 'output')
# IMGSZ = 640

# class countblood():
#     def __init__(self, path, devicenum=-1):
#         self.path = path
#         self.devicenum = devicenum
#         if self.devicenum == -1:
#             print('CPU selected.')
#             self.devicenum = 'cpu'
#         elif torch.cuda.is_available() and devicenum < torch.cuda.device_count():
#             print('Selected GPU available.')
#             print('GPU number : ', devicenum)
#         else:
#             i = input('The selected device is not available. Use CPU? ([y]/n) ')
#             if i == 'y' or i == '':
#                 print('CPU selected.')
#                 devicenum = 'cpu'
#             else:
#                 print('No available device selected.')
#                 print('Program exit.')
#                 exit()
#         # Load qc model
#         qcmodel = torch.load(QPATH, map_location=torch.device(devicenum))
#         qcmodel.eval()

#         # Create necessary folders
#         if os.path.exists(PATH):
#             try:iname = PATH.split('/')[-1]
#             except:iname = PATH.split('\\')[-1]
#             iname = iname[:-5]
#             IPATH = os.path.join(ipath, iname)
#             DPATH = os.path.join(IPATH, 'discard')
#             try:os.makedirs(IPATH)
#             except:pass
#             try:os.makedirs(DPATH)
#             except:pass
#             try:os.makedirs(OPATH)
#             except:pass
#         else:
#             print('NDPI file does not exist. Please check the path.')
#             return 0

#         # If ndpi file exists
#         qclist = [0, 0, 0]
#         counts = []
#         total_count = [0 for _ in range(10)]
#         width, height = dimensions(PATH)

#         # tqdm progress bar for image cut and prediction
#         total_progress = len(range(0, width, IMGSZ)) * len(range(0, height, IMGSZ))
#         progress_bar = tqdm(total=total_progress, desc='Cutting image')

#         # Cut ndpi into 640px size small images
#         for i in range(0, width, IMGSZ):
#             for j in range(0, height, IMGSZ):
#                 outimg = cutimg(PATH, i, j, IMGSZ)
#                 isStandard = classify(qcmodel, outimg, devicenum)
#                 qclist[isStandard] += 1

#                 progress_bar.update(1)

#                 # Save image if standard, discard if not standard
#                 if isStandard != 2:
#                     # saveimg(DPATH, outimg, i, j)
#                     continue
#                 counts.append(predict(MODEL, outimg, devicenum))
    
#         # tqdm progress bar for counting
#         total_progress = len(counts)
#         progress_bar = tqdm(total=total_progress, desc='Counting cells')
#         for count in counts:
#             for c in zip(count[0], count[1]):
#                 if int(c[0]) == 10: continue
#                 total_count[int(c[0])] += int(c[1])
#             progress_bar.update(1)
        
#         output(OPATH, iname, total_count, qclist)